VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CTrayClear"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'****************************************************
'* CTrayClear.cls                                   *
'* clears the system tray of abandonned icons       *
'* Programmed: Konrad Rudolph (Mad Rat Pro.)        *
'* Last Change: 06.08.2003 23:38                    *
'* Version: 1.0.4                                   *
'****************************************************
Option Explicit

Const CLASSNAME As String = "CTrayClear"
'----------------------------------------------------

' API declarations
Private Declare Function SendMessage Lib "user32" Alias "SendMessageA" ( _
  ByVal hWnd As hWnd, _
  ByVal wMsg As Long, _
  ByVal wParam As Long, _
  ByRef lParam As Any _
) As Long
Private Declare Function FindWindowEx Lib "user32" Alias "FindWindowExA" ( _
  ByVal hWndParent As hWnd, _
  ByVal hWndChildAfter As hWnd, _
  ByVal lpszClass As String, _
  ByVal lpszWindow As String _
) As hWnd
Private Declare Function GetDesktopWindow Lib "user32" ( _
) As hWnd
Private Declare Function GetClientRect Lib "user32" ( _
  ByVal hWnd As hWnd, _
  ByRef lpRect As RECT _
) As Long
'----------------------------------------------------

Private Const WM_MOUSEMOVE As Long = &H200
'----------------------------------------------------

Private Type RECT
    Left    As Long
    Top     As Long
    Right   As Long
    Bottom  As Long
End Type

' Following "senseless" enum provides VB with a (fake) hWnd-datatype
Private Enum hWnd
    [_]
End Enum
'----------------------------------------------------

' Member variables
Private m_sErrorSource  As String
Private m_sErrorDesc    As String
Private m_lErrorNumber  As Long
'----------------------------------------------------

' Properties

' returns the source of an error, if any
Public Property Get ErrorSource() As String
    ErrorSource = m_sErrorSource
End Property

' returns the description of the error, if any
Public Property Get ErrorDescription() As String
    ErrorDescription = m_sErrorDesc
End Property

' returns error number (0 if there is none)
Public Property Get ErrorNumber() As Long
    ErrorNumber = m_lErrorNumber
End Property
'----------------------------------------------------

' Public procedures

'----------------------------------------------------
' DoClear As Boolean
' performs clearing of system tray
' arguments:
' - [IN] [ClearAll As Boolean]: slower, but clears everything
' - returns success (get error description via member variables)
'----------------------------------------------------
Public Function DoClear(Optional ClearAll As Boolean = False) As Boolean
    Const PROCNAME  As String = "DoClear"
    '------------------------------------------------
    Dim hWndTray    As hWnd     ' hWnd of system tray
    Dim RectTray    As RECT     ' coordinates of system tray
    Dim lRows       As Long, _
        lCols       As Long     ' number of rows and columns of icons
    
    On Error GoTo ERR_DoClear
    
    ' clear error description
    Call ClearError
    
    ' get systray hWnd
    hWndTray = GetSystrayWnd()
    
    If Not CBool(hWndTray) Then
        Call Err.Raise( _
            vbObjectError, _
            CLASSNAME & "::" & PROCNAME, _
            "Fatal Error: Systray window not found." _
        )
    End If ' Not CBool(hWndTray)
    
    ' resolve window's coordinates
    Call GetClientRect(hWndTray, RectTray)
    
    ' calculate approx. number of icons in tray,
    ' assuming that each icon is 16*16 pixels in size
    lRows = (RectTray.Bottom - RectTray.Top) \ 16
    lCols = (RectTray.Right - RectTray.Left) \ 16
    
    ' clear (all) icons
    ' -> see note in accompanying text file!
    If ClearAll Then _
        Call ClearAllIcons( _
            lRows * lCols, _
            hWndTray, _
            lCols _
        ) _
    Else _
        Call ClearIcons( _
            hWndTray, lCols, lRows _
        )
    
    ' return success
    DoClear = True
    On Error GoTo 0
    Exit Function

ERR_DoClear:
    ' fill error description
    m_lErrorNumber = Err.Number
    m_sErrorSource = Err.Source
    m_sErrorDesc = Err.Description
End Function
'----------------------------------------------------

' Private procedures

'----------------------------------------------------
' ClearAllIcons
' deletes all abandonned tray icons
' for explanation see note in accompanying text file
' arguments:
' - [IN] IconCount As Long: maximum number of icons in tray
' - [IN] hWndTarget As hWnd: handle to system tray
' - [IN] Cols As Long: number of columns of icons in tray
'----------------------------------------------------
Private Sub ClearAllIcons( _
  IconCount As Long, _
  hWndTarget As hWnd, _
  Cols As Long _
)
    Dim Idx As Long
    Dim Cnt As Long
    Dim X   As Long, _
        Y   As Long
    
    ' do the clearing loop once for each icon.
    For Cnt = 1 To IconCount
        ' loop through all icons
        ' and simulate mouse moves
        ' - coordinates are shifted by 8 pixels each to
        ' position the virtual mouse pointer in the centre
        For Idx = 1 To Cnt
            X = (Idx - 1) Mod Cols
            Y = (Idx - 1) \ Cols
            
            Call SimulateMouseMove( _
                hWndTarget, _
                X * 16 + 8, _
                Y * 16 + 8 _
            )
        Next Idx
    Next Cnt
End Sub

'----------------------------------------------------
' ClearIcons
' deletes abandonned tray icons
' for explanation see note in accompanying text file
' arguments:
' - [IN] hWndTarget As hWnd: handle to system tray
' - [IN] Cols As Long: number of columns of icons in tray
' - [IN] Rows As Long: number of rows of icons in tray
'----------------------------------------------------
Private Sub ClearIcons( _
  hWndTarget As hWnd, _
  Cols As Long, _
  Rows As Long _
)
    Dim X   As Long, _
        Y   As Long
    
    ' loop through all icons
    ' and simulate mouse moves
    ' - coordinates are shifted by 8 pixels each to
    ' position the virtual mouse pointer in the centre
    For X = 0 To Cols - 1
        For Y = 0 To Rows - 1
            Call SimulateMouseMove( _
                hWndTarget, _
                X * 16 + 8, _
                Y * 16 + 8 _
            )
        Next Y
    Next X
End Sub

'----------------------------------------------------
' ClearError
' clears error description
'----------------------------------------------------
Private Sub ClearError()
    m_lErrorNumber = 0
    m_sErrorSource = ""
    m_sErrorDesc = ""
End Sub

'----------------------------------------------------
' GetSystrayWnd As hWnd
' - returns the hWnd of the systray window or null on failure
'----------------------------------------------------
Private Function GetSystrayWnd() As hWnd
    ' define window class names for different systray parent windows
    Const CLASS_TASKBAR As String = "Shell_TrayWnd"     ' taskbar
    Const CLASS_TRAYWND As String = "TrayNotifyWnd"     ' tray notification area
    Const CLASS_SYSPAGE As String = "SysPager"          ' systray area
    Const CLASS_TOOLBAR As String = "ToolbarWindow32"   ' tray icon bar
    '------------------------------------------------
    Dim hWndDesktop As hWnd
    Dim hWndTemp    As hWnd
    
    hWndDesktop = GetDesktopWindow()    ' get the desktop windows's hWnd
    
    ' taskbar hWnd
    hWndTemp = FindWindowEx( _
        hWndDesktop, 0&, _
        CLASS_TASKBAR, _
        vbNullString _
    )
    If Not CBool(hWndTemp) Then _
        Exit Function                   ' taskbar not found
    
    ' tray notification area hWnd
    hWndTemp = FindWindowEx( _
        hWndTemp, 0&, _
        CLASS_TRAYWND, _
        vbNullString _
    )
    If Not CBool(hWndTemp) Then _
        Exit Function                   ' taskbar not found
    
    ' systray hWnd
    hWndTemp = FindWindowEx( _
        hWndTemp, 0&, _
        CLASS_SYSPAGE, _
        vbNullString _
    )
    If Not CBool(hWndTemp) Then _
        Exit Function                   ' taskbar not found
    
    ' tray icon bar hWnd --> we need this one!
    GetSystrayWnd = FindWindowEx( _
        hWndTemp, 0&, _
        CLASS_TOOLBAR, _
        vbNullString _
    )
End Function

'----------------------------------------------------
' SimulateMouseMove
' sends a WM_MOUSEMOVE message to a target window
' arguments:
' - [IN] hWndTarget As hWnd: hWnd of target window
' - [IN] ClientX As Long: virtual X-position of mouse pointer
' - [IN] ClientY As Long: virtual Y-position of mouse pointer
'----------------------------------------------------
Private Sub SimulateMouseMove( _
  hWndTarget As hWnd, _
  ClientX As Long, _
  ClientY As Long _
)
    Const PROCNAME  As String = "SimulateMouseMove"
    '------------------------------------------------
    Dim lParamPos   As Long
    Dim lResult     As Long
    
    ' lParam for the WM_MOUSEMOVE message:
    ' 2 low-order bytes: x position
    ' 2 high-order bytes: y position
    lParamPos = ClientY * 2 ^ 16 + ClientX
    
    ' send WM_MOUSEMOVE message
    lResult = SendMessage( _
        hWndTarget, _
        WM_MOUSEMOVE, _
        0&, ByVal lParamPos _
    )
    
    If lResult <> 0 Then
        ' call failed
        Call Err.Raise( _
            vbObjectError, _
            CLASSNAME & "::" & PROCNAME, _
            "SendMessage() failed for window &H" & _
            Hex$(hWndTarget) & "." _
        )
    End If ' lResult <> 0
End Sub
